#include "Pathfinder.h"

Pathfinder::Pathfinder()
{
}

Pathfinder::~Pathfinder(void)
{
    ResetPath();
}

bool Pathfinder::GetValidPath(RPG::Event *event, Point<UINT> uiEndTile, bool bFindNearestPoint)
{
    if (!event)
        return false;

    Point<UINT> uiStartTile(event->x, event->y);
    // return if start pos and end pos are the same tile
    if (uiStartTile == uiEndTile)
        return false;

    Point<UINT> range = GetRangeBetween(uiStartTile, uiEndTile);
    // add starting point to closed nodes.
    PathNode *CurNode = new PathNode(uiStartTile, PATH_NONE, 1, range.x*TILE_SIZE_X + range.y*TILE_SIZE_Y, NULL);
    m_ClosedNodes.insert(std::make_pair<Point<UINT>, PathNode*>(CurNode->m_uiPos, CurNode));

    bool bFinishPath = false;
    while(!bFinishPath)
    {
        // check tiles around tile
        for (USHORT i = 0; i < 4; i++)
        {
            PathNode *NewNode = NULL;
            switch(i)
            {
                // top
            case 0:
                if (!event->isMovePossible(CurNode->m_uiPos.x, CurNode->m_uiPos.y, CurNode->m_uiPos.x, CurNode->m_uiPos.y-1))
                    continue;

                range = GetRangeBetween(Point<UINT>(CurNode->m_uiPos.x, CurNode->m_uiPos.y -1), uiEndTile);
                NewNode = new PathNode(Point<UINT>(CurNode->m_uiPos.x, CurNode->m_uiPos.y -1), PATH_UP, CurNode->m_uiGPrice + 1, range.x*TILE_SIZE_X + range.y*TILE_SIZE_Y, CurNode);
                break;
                // left
            case 1:
                if (!event->isMovePossible(CurNode->m_uiPos.x, CurNode->m_uiPos.y, CurNode->m_uiPos.x-1, CurNode->m_uiPos.y))
                    continue;

                range = GetRangeBetween(Point<UINT>(CurNode->m_uiPos.x -1, CurNode->m_uiPos.y), uiEndTile);
                NewNode = new PathNode(Point<UINT>(CurNode->m_uiPos.x -1, CurNode->m_uiPos.y), PATH_LEFT, CurNode->m_uiGPrice + 1, range.x*TILE_SIZE_X + range.y*TILE_SIZE_Y, CurNode);
                break;
                // bottom
            case 2:
                if (!event->isMovePossible(CurNode->m_uiPos.x, CurNode->m_uiPos.y, CurNode->m_uiPos.x, CurNode->m_uiPos.y+1))
                    continue;

                range = GetRangeBetween(Point<UINT>(CurNode->m_uiPos.x, CurNode->m_uiPos.y +1), uiEndTile);
                NewNode = new PathNode(Point<UINT>(CurNode->m_uiPos.x, CurNode->m_uiPos.y +1), PATH_DOWN, CurNode->m_uiGPrice + 1, range.x*TILE_SIZE_X + range.y*TILE_SIZE_Y, CurNode);
                break;
                // right
            case 3:
                if (!event->isMovePossible(CurNode->m_uiPos.x, CurNode->m_uiPos.y, CurNode->m_uiPos.x+1, CurNode->m_uiPos.y))
                    continue;

                range = GetRangeBetween(Point<UINT>(CurNode->m_uiPos.x +1, CurNode->m_uiPos.y), uiEndTile);
                NewNode = new PathNode(Point<UINT>(CurNode->m_uiPos.x+1, CurNode->m_uiPos.y), PATH_RIGHT, CurNode->m_uiGPrice + 1, range.x*TILE_SIZE_X + range.y*TILE_SIZE_Y, CurNode);
                break;
            default:
                continue;
            }

            if (IsStoredInClosedList(NewNode))
            {
                delete NewNode;
                continue;
            }
            else if (IsStoredInOpenList(NewNode))
            {
                if (PathNode *tempNode = GetStoredPathNode(NewNode->m_uiPos))
                {
                    if (NewNode->m_uiGPrice < tempNode->m_uiGPrice)
                    {
                        *tempNode = *NewNode;
                        delete NewNode;
                    }
                }
            }
            else
                m_OpenNodes.insert(std::make_pair<Point<UINT>, PathNode*>(NewNode->m_uiPos, NewNode));
        }

        // get PathNode with lowest F
        NodeList::iterator storedItr = m_OpenNodes.end();
        UINT uiLowestF = 0-1;
        if (!m_OpenNodes.empty())
        {
            for (NodeList::iterator itr = m_OpenNodes.begin(); itr != m_OpenNodes.end(); ++itr)
            {
                if (!itr->second)
                    continue;

                // if F lower than stored F, store curNode
                if (itr->second->m_uiGPrice + itr->second->m_uiHCalc < uiLowestF)
                {
                    uiLowestF = itr->second->m_uiGPrice + itr->second->m_uiHCalc;
                    storedItr = itr;
                }
            }
        }
        // if open nodes is empty get node with lowest f from closed list or return false
        else
        {
            if (bFindNearestPoint)
            {
                storedItr = m_ClosedNodes.end();
                for (NodeList::iterator itr = m_ClosedNodes.begin(); itr != m_ClosedNodes.end(); ++itr)
                {
                    if (!itr->second)
                        continue;

                    // if F lower than stored F, store curNode
                    if (itr->second->m_uiGPrice + itr->second->m_uiHCalc < uiLowestF)
                    {
                        uiLowestF = itr->second->m_uiGPrice + itr->second->m_uiHCalc;
                        storedItr = itr;
                    }
                }
                // if find a point break loop an read out path
                if (storedItr != m_ClosedNodes.end())
                {
                    CurNode = storedItr->second;
                    break;
                }
            }

            return false;
        }

        // switch node from openlist to closed list
        if (storedItr != m_OpenNodes.end())
        {
            CurNode = storedItr->second;
            m_OpenNodes.erase(storedItr);
            m_ClosedNodes.insert(std::make_pair<Point<UINT>, PathNode*>(CurNode->m_uiPos, CurNode));
        }

        if (CurNode->m_uiPos == uiEndTile)
            bFinishPath = true;
    }

    // read out of path
    m_Path.clear();
    while(CurNode)
    {
        m_Path.push_front(CurNode->m_PathDir);
        CurNode = CurNode->m_pLastNode;
    }

    return true;
}

void Pathfinder::ResetPath()
{
    // delete open nodes
    for (NodeList::iterator itr = m_OpenNodes.begin(); itr != m_OpenNodes.end(); ++itr)
    {
        if (itr->second)
            delete itr->second;
    }
    m_OpenNodes.clear();

    // delete closed nodes
    for (NodeList::iterator itr = m_ClosedNodes.begin(); itr != m_ClosedNodes.end(); ++itr)
    {
        if (itr->second)
            delete itr->second;
    }
    m_ClosedNodes.clear();
}

bool Pathfinder::IsStoredInClosedList(PathNode *Node)
{
    NodeList::iterator itr = m_ClosedNodes.find(Node->m_uiPos);
    if (itr != m_ClosedNodes.end())
        return true;

    return false;
}

bool Pathfinder::IsStoredInOpenList(PathNode *Node)
{
    NodeList::iterator itr = m_OpenNodes.find(Node->m_uiPos);
    if (itr != m_OpenNodes.end())
        return true;

    return false;
}

PathNode* Pathfinder::GetStoredPathNode(Point<UINT> uiTile)
{
    NodeList::iterator itr = m_OpenNodes.find(uiTile);
    if (itr != m_OpenNodes.end())
        return itr->second;

    itr = m_ClosedNodes.find(uiTile);
    if (itr != m_ClosedNodes.end())
        return itr->second;

    return NULL;
}

Point<UINT> Pathfinder::GetRangeBetween(Point<UINT> start, Point<UINT> end)
{
    Point<UINT> range;
    if (start.x > end.x)
        range.x = start.x - end.x;
    else
        range.x = end.x - start.x;

    if (start.y > end.y)
        range.y = start.y - end.y;
    else
        range.y = end.y - start.y;

    return range;
}

void Pathfinder::MoveByPath(RPG::Event *event, UINT uiMoveSpeed, UINT uiFrequency, UINT uiMaxTilesToMove, std::string sAddToMove)
{
    // move by path
    std::string moves = "";
    UINT uiMoves = 0;
    for (std::list<PathDir>::iterator itr = m_Path.begin(); itr != m_Path.end(); ++itr)
    {
        switch(*itr)
        {
        case PATH_UP:
            moves += (char)RPG::MT_MOVE_UP;
            uiMoves++;
            break;
        case PATH_DOWN:
            moves += (char)RPG::MT_MOVE_DOWN;
            uiMoves++;
            break;
        case PATH_LEFT:
            moves += (char)RPG::MT_MOVE_LEFT;
            uiMoves++;
            break;
        case PATH_RIGHT:
            moves += (char)RPG::MT_MOVE_RIGHT;
            uiMoves++;
            break;
        default:
            break;
        }

        if (uiMaxTilesToMove != 0 && uiMoves >= uiMaxTilesToMove)
            break;
    }
    moves += sAddToMove;

    event->stop();
    event->move(moves.c_str(), moves.length(), false, false, uiFrequency);
    event->speed = uiMoveSpeed;
}
