#ifndef PATHFINDER_H
#define PATHFINDER_H

#define NOT_MAIN_MODULE
#include <DynRPG/DynRPG.h>
#include "Point.h"
#include <map>
#include <list>

class Unit;

const UINT TILE_SIZE_X = 16;
const UINT TILE_SIZE_Y = 16;

enum PathDir
{
    PATH_NONE,
    PATH_UP,
    PATH_LEFT,
    PATH_DOWN,
    PATH_RIGHT,
};

struct PathNode
{
    PathNode(Point<UINT> uiPos, PathDir pathDir, UINT uiGPrice, UINT uiHCalc, PathNode *LastNode) : m_PathDir(pathDir), m_pLastNode(LastNode), m_uiPos(uiPos),
                m_uiGPrice(uiGPrice), m_uiHCalc(uiHCalc)
    { }

    PathDir m_PathDir;
    PathNode *m_pLastNode;
    Point<UINT> m_uiPos;
    UINT m_uiGPrice;
    UINT m_uiHCalc;
};

typedef std::map<Point<UINT>, PathNode*> NodeList;

class Pathfinder
{
public:
    Pathfinder();
    ~Pathfinder(void);

    bool GetValidPath(RPG::Event *event, Point<UINT> uiEndPos, bool bFindNearestPoint);
    Point<UINT> GetRangeBetween(Point<UINT> start, Point<UINT> end);
    void MoveByPath(RPG::Event *event, UINT uiMoveSpeed, UINT uiFrequency, UINT uiMaxTilesToMove, std::string sAddToMove);

private:
    bool IsStoredInClosedList(PathNode *Node);
    bool IsStoredInOpenList(PathNode *Node);
    void ResetPath();
    PathNode* GetStoredPathNode(Point<UINT> uiTile);

    NodeList m_OpenNodes;
    NodeList m_ClosedNodes;
    std::list<PathDir> m_Path;
};
#endif
