#define AUTO_DLLMAIN
#include <DynRPG/DynRPG.h>
#include "Pathfinder.h"

// called on comment
bool onComment(const char *text, const RPG::ParsedCommentData *parsedData, RPG::EventScriptLine *nextScriptLine, RPG::EventScriptData *scriptData, int eventId,
int pageId, int lineId, int *nextLineId)
{
    // calculate path from obj to point
    if(strcmp(parsedData->command, "findpath") == 0
       || strcmp(parsedData->command, "findpathornearestspot") == 0)
    {
        /* param 0 = ID (value or keyword (hero, this)
        param 1 = endX
        param 2 = endY
        param 3 = movement speed
        param 4 = move frequence
        param 5 = max tiles to move
        param 6 = result variable */
        if (!RPG::map)
            return false;

        Pathfinder path;
        Point<UINT> uiEndPoint;
        RPG::Event *event = NULL;
        UINT uiParam = 0;
        std::string sAddToMove = "";

        // if next line is "set event location"
        if (nextScriptLine && nextScriptLine->command == RPG::EVCMD_SET_EVENT_LOCATION)
        {
            UINT uiEventID = (UINT)nextScriptLine->parameters[0];
            if (uiEventID == 10005)
                uiEventID = eventId;

            event = RPG::map->events[uiEventID];

            // if place is set normal
            if (nextScriptLine->parameters[1] == 0)
            {
                uiEndPoint.x = (UINT)nextScriptLine->parameters[2];
                uiEndPoint.y = (UINT)nextScriptLine->parameters[3];
            }
            // if place is set by variable
            else
            {
                uiEndPoint.x = RPG::variables[nextScriptLine->parameters[2]];
                uiEndPoint.y = RPG::variables[nextScriptLine->parameters[3]];
            }

            // if facing is set
            switch(nextScriptLine->parameters[4])
            {
            case 1:
                sAddToMove = (char)RPG::MT_FACE_UP;
                break;
            case 2:
                sAddToMove = (char)RPG::MT_FACE_RIGHT;
                break;
            case 3:
                sAddToMove = (char)RPG::MT_FACE_DOWN;
                break;
            case 4:
                sAddToMove = (char)RPG::MT_FACE_LEFT;
                break;
            default:
                break;
            }
            *nextLineId = scriptData->currentLineId + 1;
        }
        // if koords are in comments
        else if (parsedData->parametersCount >= 3 && parsedData->parameters[1].type == RPG::PARAM_NUMBER && parsedData->parameters[2].type == RPG::PARAM_NUMBER)
        {
            // check endpos; if out of map, return
            if (parsedData->parameters[1].number > RPG::map->getWidth()-1 || parsedData->parameters[1].number < 0 ||
                parsedData->parameters[2].number > RPG::map->getHeight()-1 || parsedData->parameters[2].number < 0)
                return false;

            uiEndPoint.x = (UINT)parsedData->parameters[1].number;
            uiEndPoint.y = (UINT)parsedData->parameters[2].number;

            // check target
            switch(parsedData->parameters[0].type)
            {
            case RPG::PARAM_NUMBER:
                if (parsedData->parameters[0].number > 0)
                    event = RPG::map->events[parsedData->parameters[0].number];
                else
                    return false;
                break;
            case RPG::PARAM_TOKEN:
                // set ID of event which calls
                if (strcmp(parsedData->parameters[0].text, "this") == 0)
                    event = RPG::map->events[eventId];
                // set ID to hero
                else if (strcmp(parsedData->parameters[0].text, "hero") == 0)
                    event = (RPG::Event*)RPG::hero;
                break;
            default:
                return false;
            }

            uiParam = 3;
        }
        else
            return false;

        // movement speed
        UINT uiSpeed = 3;
        if ((UINT)parsedData->parametersCount > uiParam && parsedData->parameters[uiParam].type == RPG::PARAM_NUMBER)
        {
            uiSpeed = (UINT)parsedData->parameters[uiParam].number;
            // wrap speed to valid value
            if (uiSpeed > 6)
                uiSpeed = 6;
            else if (uiSpeed < 1)
                uiSpeed = 1;
        }
        else if ((UINT)parsedData->parametersCount >= uiParam && parsedData->parameters[uiParam].type == RPG::PARAM_TOKEN && strcmp(parsedData->parameters[uiParam].text, "default") == 0)
            uiSpeed = event->speed;

        // movement frequency
        UINT uiFrequency = 8;
        if ((UINT)parsedData->parametersCount > uiParam && parsedData->parameters[uiParam+1].type == RPG::PARAM_NUMBER)
        {
            uiFrequency = (UINT)parsedData->parameters[uiParam+1].number;
            // wrap speed to valid value
            if (uiFrequency > 8)
                uiFrequency = 8;
            else if (uiFrequency < 1)
                uiFrequency = 1;
        }
        else if ((UINT)parsedData->parametersCount >= uiParam && parsedData->parameters[uiParam+1].type == RPG::PARAM_TOKEN && strcmp(parsedData->parameters[uiParam+1].text, "default") == 0)
            uiFrequency = event->frequency;

        // Max Tiles to Move
        UINT uiMaxTilesToMove = 0;
        if ((UINT)parsedData->parametersCount > uiParam && parsedData->parameters[uiParam+2].type == RPG::PARAM_NUMBER)
            uiMaxTilesToMove = (UINT)parsedData->parameters[uiParam+2].number;

        // find path
        bool bSuccess = false;
        if (strcmp(parsedData->command, "findpath") == 0)
            bSuccess = path.GetValidPath(event, uiEndPoint, false);
        else if (strcmp(parsedData->command, "findpathornearestspot") == 0)
            bSuccess = path.GetValidPath(event, uiEndPoint, true);

        // if path found, move
        if (bSuccess)
            path.MoveByPath(event, uiSpeed, uiFrequency, uiMaxTilesToMove, sAddToMove);

        // result variable
        if ((UINT)parsedData->parametersCount >= uiParam && parsedData->parameters[uiParam+3].type == RPG::PARAM_NUMBER && parsedData->parameters[uiParam+3].number > 0)
            RPG::switches[parsedData->parameters[uiParam+3].number] = bSuccess;

        return false;
    }   // if(strcmp(parsedData->command, "calcpath") == 0)

    // return ID of current event
    else if(strcmp(parsedData->command, "getcureventid") == 0)
    {
        if (parsedData->parametersCount >= 1 && parsedData->parameters[0].type == RPG::PARAM_NUMBER)
            RPG::variables[(UINT)parsedData->parameters[0].number] = eventId;

        return false;
    }   // if(strcmp(parsedData->command, "getcureventid") == 0)

    // wait until event not moving
    else if(strcmp(parsedData->command, "waituntilmove") == 0)
    {
        RPG::Event *event = NULL;
        // paramter 1 = event ID
        if (parsedData->parametersCount == 1 && parsedData->parameters[0].type == RPG::PARAM_NUMBER)
            event = RPG::map->events[(int)parsedData->parameters[0].number];

        // parameter 1 = keyword
        else if (parsedData->parameters[0].type == RPG::PARAM_TOKEN)
        {
            if (strcmp(parsedData->parameters[0].text, "this") == 0)
                event = RPG::map->events[eventId];

            else if (strcmp(parsedData->parameters[0].text, "hero") == 0)
                event = (RPG::Event*)RPG::hero;
        }

        // if no paramter, use current event
        else if (parsedData->parametersCount == 0)
            event = RPG::map->events[eventId];

        if (event && event->moving)
        {
        }

        return false;
    }   // if(strcmp(parsedData->command, "waituntilmove") == 0)

    return true;
}
