/*
	Date & Time
	A plugin for RPG Maker 2003
	Using DynRPG - the RM2k3 Plugin SDK

	This plugin allows you to get the current date and time and work with timestamps.

	By elvissteinjr
	Some code and comments are based on the keyboard_mouse_input plugin by Cherry.
*/

#define AUTO_DLLMAIN
#include <DynRPG/DynRPG.h>
#include <time.h>

// This handler is called on startup
bool onStartup(	char * 	pluginName  )
{
    //This will remove the variable limits for the Change Variable event command
    //We need to do is to be able to calculate with timestamps in event code.
    *reinterpret_cast<unsigned char *>(0x4AD36F) = 0xEB;
    *reinterpret_cast<unsigned char *>(0x4AD399) = 0xEB;

    return true; // Don't forget to return true so that the start of the game will continue!
}

// This handler is called when an event comment was encountered
bool onComment(const char *text, const RPG::ParsedCommentData *parsedData,
	RPG::EventScriptLine *nextScriptLine, RPG::EventScriptData *scriptData,
	int	eventId, int pageId, int lineId, int *nextLineId)
{
    if(strcmp(parsedData->command, "gettimestamp") == 0)
    {
		// Next, we make sure that the parameters are valid
		// (one numerical parameter).
		// Parameter 0 is the ID of the variable we should write
		// the result to.
		if(parsedData->parametersCount == 1 &&
			parsedData->parameters[0].type == RPG::PARAM_NUMBER)
		{
			// We load the parameter to an integer
			int variableId = (int)parsedData->parameters[0].number;

            // Then we set the time
            RPG::variables[variableId] = time(NULL);
        }
		else
		{
			// Show an error if the parameters were invalid
			RPG::showError("Parameters invalid for GetTimestamp function!", eventId, pageId, lineId);
		}
		return false; // We handled this command, so we return false!
    }
    else if(strcmp(parsedData->command, "gettime") == 0)
    {
        // Next, we make sure that the parameters are valid
		// (three numerical parameters).
		// Parameters 0-2 is the ID of the variables we should write
		// the result to.
		if(parsedData->parametersCount == 3 &&
			parsedData->parameters[0].type == RPG::PARAM_NUMBER &&
            parsedData->parameters[1].type == RPG::PARAM_NUMBER &&
            parsedData->parameters[2].type == RPG::PARAM_NUMBER)
		{
            time_t t;           //Time as timestamp
            struct tm *ts;      //Time as struct

            t = time(NULL);     //We get the current timestamp...
            ts = localtime(&t); //And make the struct out if it, which now contains more detailed information about it


			// We load the first parameter to an integer
			int variableId = (int)parsedData->parameters[0].number;

            // Then we set the time
            RPG::variables[variableId] = ts->tm_hour;

            // We load the second parameter to an integer
			variableId = (int)parsedData->parameters[1].number;

            // Then we set the time
            RPG::variables[variableId] = ts->tm_min;

            // We load the third parameter to an integer
			variableId = (int)parsedData->parameters[2].number;

            // Then we set the time
            RPG::variables[variableId] = ts->tm_sec;
        }
		else
		{
			// Show an error if the parameters were invalid
			RPG::showError("Parameters invalid for GetTime function!", eventId, pageId, lineId);
		}
		return false; // We handled this command, so we return false!
    }
    else if(strcmp(parsedData->command, "gethour") == 0)
    {
        // Next, we make sure that the parameters are valid
		// (one numerical parameter).
		// Parameter 0 is the ID of the variable we should write
		// the result to.
		if(parsedData->parametersCount == 1 &&
			parsedData->parameters[0].type == RPG::PARAM_NUMBER)
		{
            time_t t;           //Time as timestamp
            struct tm *ts;      //Time as struct

            t = time(NULL);     //We get the current timestamp...
            ts = localtime(&t); //And make the struct out if it, which now contains more detailed information about it


			// We load the parameter to an integer
			int variableId = (int)parsedData->parameters[0].number;

            // Then we set the time
            RPG::variables[variableId] = ts->tm_hour;
        }
		else
		{
			// Show an error if the parameters were invalid
			RPG::showError("Parameters invalid for GetHour function!", eventId, pageId, lineId);
		}
		return false; // We handled this command, so we return false!
    }
    else if(strcmp(parsedData->command, "getminute") == 0)
    {
        // Next, we make sure that the parameters are valid
		// (one numerical parameter).
		// Parameter 0 is the ID of the variable we should write
		// the result to.
		if(parsedData->parametersCount == 1 &&
			parsedData->parameters[0].type == RPG::PARAM_NUMBER)
		{
            time_t t;           //Time as timestamp
            struct tm *ts;      //Time as struct

            t = time(NULL);     //We get the current timestamp...
            ts = localtime(&t); //And make the struct out if it, which now contains more detailed information about it


			// We load the parameter to an integer
			int variableId = (int)parsedData->parameters[0].number;

            // Then we set the time
            RPG::variables[variableId] = ts->tm_min;
        }
		else
		{
			// Show an error if the parameters were invalid
			RPG::showError("Parameters invalid for GetMinute function!", eventId, pageId, lineId);
		}
		return false; // We handled this command, so we return false!
    }
    else if(strcmp(parsedData->command, "getsecond") == 0)
    {
        // Next, we make sure that the parameters are valid
		// (one numerical parameter).
		// Parameter 0 is the ID of the variable we should write
		// the result to.
		if(parsedData->parametersCount == 1 &&
			parsedData->parameters[0].type == RPG::PARAM_NUMBER)
		{
            time_t t;           //Time as timestamp
            struct tm *ts;      //Time as struct

            t = time(NULL);     //We get the current timestamp...
            ts = localtime(&t); //And make the struct out if it, which now contains more detailed information about it


			// We load the parameter to an integer
			int variableId = (int)parsedData->parameters[0].number;

            // Then we set the time
            RPG::variables[variableId] = ts->tm_sec;
        }
		else
		{
			// Show an error if the parameters were invalid
			RPG::showError("Parameters invalid for GetSecond function!", eventId, pageId, lineId);
		}
		return false; // We handled this command, so we return false!
    }
    else if(strcmp(parsedData->command, "getdate") == 0)
    {
        // Next, we make sure that the parameters are valid
		// (three numerical parameters).
		// Parameters 0-2 is the ID of the variables we should write
		// the result to.
		if(parsedData->parametersCount == 3 &&
			parsedData->parameters[0].type == RPG::PARAM_NUMBER &&
            parsedData->parameters[1].type == RPG::PARAM_NUMBER &&
            parsedData->parameters[2].type == RPG::PARAM_NUMBER)
		{
            time_t t;           //Time as timestamp
            struct tm *ts;      //Time as struct

            t = time(NULL);     //We get the current timestamp...
            ts = localtime(&t); //And make the struct out if it, which now contains more detailed information about it


			// We load the first parameter to an integer
			int variableId = (int)parsedData->parameters[0].number;

            // Then we set the month
            RPG::variables[variableId] = ts->tm_mon + 1; //We add 1 because tm_mon starts with 0

            // We load the second parameter to an integer
			variableId = (int)parsedData->parameters[1].number;

            // Then we set the day
            RPG::variables[variableId] = ts->tm_mday;

            // We load the third parameter to an integer
			variableId = (int)parsedData->parameters[2].number;

            // Then we set the year
            RPG::variables[variableId] = ts->tm_year + 1900; //We add 1900 to get the full year number
        }
		else
		{
			// Show an error if the parameters were invalid
			RPG::showError("Parameters invalid for GetDate function!", eventId, pageId, lineId);
		}
		return false; // We handled this command, so we return false!
    }
    else if(strcmp(parsedData->command, "getmonth") == 0)
    {
        // Next, we make sure that the parameters are valid
		// (one numerical parameter).
		// Parameter 0 is the ID of the variable we should write
		// the result to.
		if(parsedData->parametersCount == 1 &&
			parsedData->parameters[0].type == RPG::PARAM_NUMBER)
		{
            time_t t;           //Time as timestamp
            struct tm *ts;      //Time as struct

            t = time(NULL);     //We get the current timestamp...
            ts = localtime(&t); //And make the struct out if it, which now contains more detailed information about it


			// We load the parameter to an integer
			int variableId = (int)parsedData->parameters[0].number;

            // Then we set the date
            RPG::variables[variableId] = ts->tm_mon + 1; //We add 1 because tm_mon starts with 0
        }
		else
		{
			// Show an error if the parameters were invalid
			RPG::showError("Parameters invalid for GetMonth function!", eventId, pageId, lineId);
		}
		return false; // We handled this command, so we return false!
    }
    else if(strcmp(parsedData->command, "getday") == 0)
    {
        // Next, we make sure that the parameters are valid
		// (one numerical parameter).
		// Parameter 0 is the ID of the variable we should write
		// the result to.
		if(parsedData->parametersCount == 1 &&
			parsedData->parameters[0].type == RPG::PARAM_NUMBER)
		{
            time_t t;           //Time as timestamp
            struct tm *ts;      //Time as struct

            t = time(NULL);     //We get the current timestamp...
            ts = localtime(&t); //And make the struct out if it, which now contains more detailed information about it


			// We load the parameter to an integer
			int variableId = (int)parsedData->parameters[0].number;

            // Then we set the date
            RPG::variables[variableId] = ts->tm_mday;
        }
		else
		{
			// Show an error if the parameters were invalid
			RPG::showError("Parameters invalid for GetDay function!", eventId, pageId, lineId);
		}
		return false; // We handled this command, so we return false!
    }
    else if(strcmp(parsedData->command, "getyear") == 0)
    {
        // Next, we make sure that the parameters are valid
		// (one numerical parameter).
		// Parameter 0 is the ID of the variable we should write
		// the result to.
		if(parsedData->parametersCount == 1 &&
			parsedData->parameters[0].type == RPG::PARAM_NUMBER)
		{
            time_t t;           //Time as timestamp
            struct tm *ts;      //Time as struct

            t = time(NULL);     //We get the current timestamp...
            ts = localtime(&t); //And make the struct out if it, which now contains more detailed information about it


			// We load the parameter to an integer
			int variableId = (int)parsedData->parameters[0].number;

            // Then we set the date
            RPG::variables[variableId] = ts->tm_year + 1900; //We add 1900 to get the full year number
        }
		else
		{
			// Show an error if the parameters were invalid
			RPG::showError("Parameters invalid for GetYear function!", eventId, pageId, lineId);
		}
		return false; // We handled this command, so we return false!
    }
    else if(strcmp(parsedData->command, "getweekday") == 0)
    {
        // Next, we make sure that the parameters are valid
		// (one numerical parameter).
		// Parameter 0 is the ID of the variable we should write
		// the result to.
		if(parsedData->parametersCount == 1 &&
			parsedData->parameters[0].type == RPG::PARAM_NUMBER)
		{
            time_t t;           //Time as timestamp
            struct tm *ts;      //Time as struct

            t = time(NULL);     //We get the current timestamp...
            ts = localtime(&t); //And make the struct out if it, which now contains more detailed information about it


			// We load the parameter to an integer
			int variableId = (int)parsedData->parameters[0].number;

            // Then we set the date
            RPG::variables[variableId] = ts->tm_wday;
        }
		else
		{
			// Show an error if the parameters were invalid
			RPG::showError("Parameters invalid for GetWeekDay function!", eventId, pageId, lineId);
		}
		return false; // We handled this command, so we return false!
    }
    else if(strcmp(parsedData->command, "getyearday") == 0)
    {
        // Next, we make sure that the parameters are valid
		// (one numerical parameter).
		// Parameter 0 is the ID of the variable we should write
		// the result to.
		if(parsedData->parametersCount == 1 &&
			parsedData->parameters[0].type == RPG::PARAM_NUMBER)
		{
            time_t t;           //Time as timestamp
            struct tm *ts;      //Time as struct

            t = time(NULL);     //We get the current timestamp...
            ts = localtime(&t); //And make the struct out if it, which now contains more detailed information about it


			// We load the parameter to an integer
			int variableId = (int)parsedData->parameters[0].number;

            // Then we set the date
            RPG::variables[variableId] = ts->tm_yday + 1; //We add 1 because tm_yday starts with 0
        }
		else
		{
			// Show an error if the parameters were invalid
			RPG::showError("Parameters invalid for GetYearDay function!", eventId, pageId, lineId);
		}
		return false; // We handled this command, so we return false!
    }
    else if(strcmp(parsedData->command, "getdatefromtimestamp") == 0)
    {
        // Next, we make sure that the parameters are valid
		// (four numerical parameters).
		// Parameter 0 is the ID of the variable we get the timestamp from
		// Parameters 1-3 are the IDs of the variables we should write
		// the result to.
		if(parsedData->parametersCount == 4 &&
			parsedData->parameters[0].type == RPG::PARAM_NUMBER &&
            parsedData->parameters[1].type == RPG::PARAM_NUMBER &&
            parsedData->parameters[2].type == RPG::PARAM_NUMBER &&
            parsedData->parameters[3].type == RPG::PARAM_NUMBER)
		{
            time_t t;           //Time as timestamp
            struct tm *ts;      //Time as struct

			//We read the timestamp from the first parameter
            t = (int)parsedData->parameters[0].number;
            ts = localtime(&t); //Then make the struct out if it, which now contains more detailed information about it

            //Now we write the date back into variables

            // We load the second parameter to an integer
			int variableId = (int)parsedData->parameters[1].number;

            // Then we set the month
            RPG::variables[variableId] = ts->tm_mon + 1; //We add 1 because tm_mon starts with 0

            // We load the third parameter to an integer
			variableId = (int)parsedData->parameters[2].number;

            // Then we set the day
            RPG::variables[variableId] = ts->tm_mday;

            // We load the fourth parameter to an integer
			variableId = (int)parsedData->parameters[3].number;

            // Then we set the year
            RPG::variables[variableId] = ts->tm_year + 1900; //We add 1900 to get the full year number
        }
		else
		{
			// Show an error if the parameters were invalid
			RPG::showError("Parameters invalid for GetDateFromTimestamp function!", eventId, pageId, lineId);
		}
		return false; // We handled this command, so we return false!
    }
    else if(strcmp(parsedData->command, "gettimefromtimestamp") == 0)
    {
        // Next, we make sure that the parameters are valid
		// (four numerical parameters).
		// Parameter 0 is the ID of the variable we get the timestamp from
		// Parameters 1-3 are the IDs of the variables we should write
		// the result to.
		if(parsedData->parametersCount == 4 &&
			parsedData->parameters[0].type == RPG::PARAM_NUMBER &&
            parsedData->parameters[1].type == RPG::PARAM_NUMBER &&
            parsedData->parameters[2].type == RPG::PARAM_NUMBER &&
            parsedData->parameters[3].type == RPG::PARAM_NUMBER)
		{
            time_t t;           //Time as timestamp
            struct tm *ts;      //Time as struct

			//We read the timestamp from the first parameter
            t = (int)parsedData->parameters[0].number;
            ts = localtime(&t); //Then make the struct out if it, which now contains more detailed information about it

            //Now we write the time back into variables

            // We load the second parameter to an integer
			int variableId = (int)parsedData->parameters[1].number;
            // Then we set the hour
            RPG::variables[variableId] = ts->tm_hour;

            // We load the third parameter to an integer
			variableId = (int)parsedData->parameters[2].number;

            // Then we set the minute
            RPG::variables[variableId] = ts->tm_min;

            // We load the fourth parameter to an integer
			variableId = (int)parsedData->parameters[3].number;

            // Then we set the second
            RPG::variables[variableId] = ts->tm_sec;
        }
		else
		{
			// Show an error if the parameters were invalid
			RPG::showError("Parameters invalid for GetTimeFromTimestamp function!", eventId, pageId, lineId);
		}
		return false; // We handled this command, so we return false!
    }
    else if(strcmp(parsedData->command, "getweekdayfromtimestamp") == 0)
    {
        // Next, we make sure that the parameters are valid
		// (two numerical parameters).
		// Parameter 0 is the ID of the variable we get the timestamp from
		// Parameter 1 is the ID of the variable we should write
		// the result to.
		if(parsedData->parametersCount == 2 &&
			parsedData->parameters[0].type == RPG::PARAM_NUMBER &&
            parsedData->parameters[1].type == RPG::PARAM_NUMBER)
		{
            time_t t;           //Time as timestamp
            struct tm *ts;      //Time as struct

			//We read the timestamp from the first parameter
            t = (int)parsedData->parameters[0].number;
            ts = localtime(&t); //Then make the struct out if it, which now contains more detailed information about it

            //Now we write the weekday back into variables

            // We load the second parameter to an integer
			int variableId = (int)parsedData->parameters[1].number;
            // Then we set the weekday
            RPG::variables[variableId] = ts->tm_wday;
        }
		else
		{
			// Show an error if the parameters were invalid
			RPG::showError("Parameters invalid for GetWeekdayFromTimestamp function!", eventId, pageId, lineId);
		}
		return false; // We handled this command, so we return false!
    }
    else if(strcmp(parsedData->command, "getyeardayfromtimestamp") == 0)
    {
        // Next, we make sure that the parameters are valid
		// (two numerical parameters).
		// Parameter 0 is the ID of the variable we get the timestamp from
		// Parameter 1 is the ID of the variable we should write
		// the result to.
		if(parsedData->parametersCount == 2 &&
			parsedData->parameters[0].type == RPG::PARAM_NUMBER &&
            parsedData->parameters[1].type == RPG::PARAM_NUMBER)
		{
            time_t t;           //Time as timestamp
            struct tm *ts;      //Time as struct

			//We read the timestamp from the first parameter
            t = (int)parsedData->parameters[0].number;
            ts = localtime(&t); //Then make the struct out if it, which now contains more detailed information about it

            //Now we write the weekday back into variables

            // We load the second parameter to an integer
			int variableId = (int)parsedData->parameters[1].number;
            // Then we set the yearday
            RPG::variables[variableId] = ts->tm_yday;
        }
		else
		{
			// Show an error if the parameters were invalid
			RPG::showError("Parameters invalid for GetYeardayFromTimestamp function!", eventId, pageId, lineId);
		}
		return false; // We handled this command, so we return false!
    }
    else if(strcmp(parsedData->command, "gettimestampfromdate") == 0)
    {
        // Next, we make sure that the parameters are valid
		// (four numerical parameters).
		// Parameter 0 is the ID of the variable we get the timestamp from
		// Parameters 1-3 are the IDs of the variables we should write
		// the result to.
		if(parsedData->parametersCount == 4 &&
			parsedData->parameters[0].type == RPG::PARAM_NUMBER &&
            parsedData->parameters[1].type == RPG::PARAM_NUMBER &&
            parsedData->parameters[2].type == RPG::PARAM_NUMBER &&
            parsedData->parameters[3].type == RPG::PARAM_NUMBER)
		{
            struct tm ts;       //Time as struct

			// We read the month from the first parameter
            ts.tm_mon = (int)parsedData->parameters[0].number - 1; //We substract 1 because it starts with 0 in the struct

            // We read the day from the second parameter
			ts.tm_mday = (int)parsedData->parameters[1].number;

            // We read the year from the third parameter
            ts.tm_year = (int)parsedData->parameters[2].number - 1900; //We substract 1900 because it's stored in the struct without it

            ts.tm_hour = 0;
            ts.tm_min  = 0;
            ts.tm_sec  = 0;


            // We load the fourth parameter to an integer
			int variableId = (int)parsedData->parameters[3].number;
            // Then we parse the data and write the timestamp into the Variable in parameter 4
            RPG::variables[variableId] = mktime(&ts);
        }
        else if(parsedData->parametersCount == 7 && //Overloaded version of GetTimestampFromDate
			parsedData->parameters[0].type == RPG::PARAM_NUMBER &&
            parsedData->parameters[1].type == RPG::PARAM_NUMBER &&
            parsedData->parameters[2].type == RPG::PARAM_NUMBER &&
            parsedData->parameters[3].type == RPG::PARAM_NUMBER &&
            parsedData->parameters[4].type == RPG::PARAM_NUMBER &&
            parsedData->parameters[5].type == RPG::PARAM_NUMBER &&
            parsedData->parameters[6].type == RPG::PARAM_NUMBER)
		{
            struct tm ts;       //Time as struct

			// We read the month from the first parameter
            ts.tm_mon = (int)parsedData->parameters[0].number - 1; //We substract 1 because it starts with 0 in the struct

            // We read the day from the second parameter
			ts.tm_mday = (int)parsedData->parameters[1].number;

            // We read the year from the third parameter
            ts.tm_year = (int)parsedData->parameters[2].number - 1900; //We substract 1900 because it's stored in the struct without it

            // We read the hour from the fourth parameter
            ts.tm_hour = (int)parsedData->parameters[3].number;

            // We read the minute from the fifth parameter
            ts.tm_min = (int)parsedData->parameters[4].number;

            // We read the second from the sixt parameter
            ts.tm_sec = (int)parsedData->parameters[5].number;

            // We load the seventh parameter to an integer
			int variableId = (int)parsedData->parameters[6].number;
            // Then we parse the data and write the timestamp into the Variable in parameter 7
            RPG::variables[variableId] = mktime(&ts);
        }
		else
		{
			// Show an error if the parameters were invalid
			RPG::showError("Parameters invalid for GetTimestampFromDate function!", eventId, pageId, lineId);
		}
		return false; // We handled this command, so we return false!
    }
    return true; // If we weren't able to handle the command, we return true!
}
